from __future__ import absolute_import, unicode_literals

from wagtail.wagtailadmin.edit_handlers import BaseChooserPanel

from wagtailmedia.widgets import AdminMediaChooser


class BaseMediaChooserPanel(BaseChooserPanel):
    object_type_name = 'media'

    @classmethod
    def widget_overrides(cls):
        return {cls.field_name: AdminMediaChooser}


class MediaChooserPanel(object):
    def __init__(self, field_name):
        self.field_name = field_name

    def bind_to_model(self, model):
        return type(str('_MediaChooserPanel'), (BaseMediaChooserPanel,), {
            'model': model,
            'field_name': self.field_name,
        })



from __future__ import unicode_literals

from django import forms
from django.forms.models import modelform_factory
from django.utils.translation import ugettext_lazy as _

from wagtail.wagtailadmin import widgets
from wagtail.wagtailadmin.forms import (
    BaseCollectionMemberForm, collection_member_permission_formset_factory
)

from wagtailmedia.models import Media
from wagtailmedia.permissions import permission_policy as media_permission_policy


class BaseMediaForm(BaseCollectionMemberForm):
    permission_policy = media_permission_policy

    def __init__(self, *args, **kwargs):
        super(BaseMediaForm, self).__init__(*args, **kwargs)

        # We will need to exclude duration, width, height, thumbnail
        # if we would get this information from files metadata

        if self.instance.type == 'audio':
            del self.fields['width']
            del self.fields['height']
            del self.fields['thumbnail']


def get_media_form(model):
    fields = model.admin_form_fields
    if 'collection' not in fields:
        # force addition of the 'collection' field, because leaving it out can
        # cause dubious results when multiple collections exist (e.g adding the
        # media to the root collection where the user may not have permission) -
        # and when only one collection exists, it will get hidden anyway.
        fields = list(fields) + ['collection']

    return modelform_factory(
        model,
        form=BaseMediaForm,
        fields=fields,
        widgets={
            'tags': widgets.AdminTagWidget,
            'file': forms.FileInput(),
            'thumbnail': forms.FileInput(),
        })


GroupMediaPermissionFormSet = collection_member_permission_formset_factory(
    Media,
    [
        ('add_media', _("Add"), _("Add/edit media you own")),
        ('change_media', _("Edit"), _("Edit any media")),
    ],
    'wagtailmedia/permissions/includes/media_permissions_formset.html'
)



from __future__ import unicode_literals

from django.test import TestCase

from wagtailmedia import models


class TestMediaQuerySet(TestCase):
    def test_search_method(self):
        # Make a test media
        media = models.Media.objects.create(title="Test media file", duration=100)

        # Search for it
        results = models.Media.objects.search("Test")
        self.assertEqual(list(results), [media])

    def test_operators(self):
        aaa_media = models.Media.objects.create(title="AAA Test media", duration=100)
        zzz_media = models.Media.objects.create(title="ZZZ Test media", duration=100)

        results = models.Media.objects.search("aaa test", operator='and')
        self.assertEqual(list(results), [aaa_media])

        results = models.Media.objects.search("aaa test", operator='or')
        sorted_results = sorted(results, key=lambda media: media.title)
        self.assertEqual(sorted_results, [aaa_media, zzz_media])

    def test_custom_ordering(self):
        aaa_media = models.Media.objects.create(title="AAA Test media", duration=100)
        zzz_media = models.Media.objects.create(title="ZZZ Test media", duration=100)

        results = models.Media.objects.order_by('title').search("Test")
        self.assertEqual(list(results), [aaa_media, zzz_media])
        results = models.Media.objects.order_by('-title').search("Test")
        self.assertEqual(list(results), [zzz_media, aaa_media])
